#include <string.h>
#include <stddef.h>
#include "shockexe.h"
#include "modsetup.h"
#include "xscommon.h"
#include "patchack.h"

/*
  time spent on interface:
  01:10
*/

/* demo */
#include "hacks1dm.h"

/* compact disk */
#include "hacks2cd.h"

#ifdef NEVER
/* CREATURE.SET file format */
#pragma pack(push, 1)
typedef struct {
  DWORD Mission3TD; /* [0,1,2] - texture detail for mission 3 only (On / Medium / Off) */
  DWORD Difficulty; /* [0,1,2] - easy, normal, hard */
  DWORD MissionMax; /* [0,1,2,3,4] - last opened mission */
  DWORD ScreenSize; /* [0,1,2,3,4] - for mission 1 and 3 (320x200, 280x175, 240x150, 200x125, 160x100) */
  DWORD SoundLevel; /* [0..15] */
  DWORD MusicLevel; /* [0..15] */
} set_item; /* 6*4 = 24 bytes */
#pragma pack(pop)
#endif

menuitem mainhack[] = {
  {"Hacks setup", NULL, 8},
  {
    "Invert mouse in flight missions",
    "Invert vertical mouse movement (on the Y-axis) for mission 1 and 3.",
    ITEM_CHKBOX
  },
  /* empty line - separator */
  {"", NULL, ITEM_GRAYED},
  {
    "Skip flight missions",
    "[cheat] Flight mission 1 and 3 will be skipped (auto-completed on start).",
    ITEM_CHKBOX
  },
  {
    "Invincibility in flight missions",
    "[cheat] Invincibility for mission 1 and 3 (flying into deadend in 3 will kill).",
    ITEM_CHKBOX
  },
  {
    "Invincibility in exploration missions",
    "[cheat] Shield will be auto-equipped on attacks regardless actual energy level.",
    ITEM_CHKBOX
  },
  {
    "Infinite Smart Bombs in exploration missions",
    "[cheat] Infinite amount of Smart Bombs for insta-kills (press [Space] to use).",
    ITEM_CHKBOX
  },
  /* empty line - separator */
  {"", NULL, ITEM_GRAYED},
  {
    "Set last mission: 5. Aboard the alien mothership",
    "[cheat] Change last available (opened) mission in the \"Play\" game menu.",
    0
  }
};

menuitem mainmssn[] = {
  {"Select mission", "Current Filename ( CREATURE.SET )", 5},
  {"1. Mission to Saturn", NULL, 0},
  {"2. Aboard the alien ship", NULL, 0},
  {"3. Mission to Tethys", NULL, 0},
  {"4. Inside the dome", NULL, 0},
  {"5. Aboard the alien mothership", NULL, 0}
};

static menuitem mainfile[] = {
  {"SHOCK.EXE game version", NULL, 3 | MENU_NFOBOX},
  {"No supported file found", NULL, ITEM_RADBOX | ITEM_GRAYED},
  {"Demo version detected", NULL, ITEM_RADBOX | ITEM_GRAYED},
  {"CD version detected", NULL, ITEM_RADBOX | ITEM_GRAYED}
};

static fix_info listhack[LIST_LEN(cdcshock)];
static fix_file fileopts[1];

static fix_info *datalist[] = {
  dmcshock,
  cdcshock
};

/* version autodetect code */
static char filename[] = "SHOCK.EXE";
static fix_file listwhat[] = {
  /* 0xAAABBBBB: AAA - offs, BBBBB - file size */
  {filename, 0x000AF0C7UL}, /* DEMO */
  {filename, 0x004AF0CBUL}  /* CDSK */
};

int DetectVersion(void) {
fix_item fcon[4];
fix_info lcon;
short i, j;
  memset(&fcon, 0, sizeof(fcon));
  memset(&lcon, 0, sizeof(lcon));
  lcon.size = LIST_LEN(fcon);
  lcon.list = fcon;
  j = 0;
  for (i = 0; i < LIST_LEN(listwhat); i++) {
    j = (listwhat[i].size >> 20);
    j &= 0xFFF;
    fcon[0].offs = 0x00064687L + j;
    listwhat[i].size &= 0x000FFFFFUL;
    for (j = 1; j < LIST_LEN(fcon); j++) {
      fcon[j].offs = fcon[0].offs + j;
    }
    /* read data bytes */
    ReadPatches(&listwhat[i], &lcon, 1);
    /* "COD#" signature */
    if (
      (fcon[0].data == 'C') && (fcon[1].data == 'I') &&
      (fcon[2].data == 'D') && (fcon[3].data == '#')
    ) {
      j = i + 1;
      break;
    }
    j = 0;
  }
  if (j) {
    i = j - 1;
    memcpy(listhack, datalist[i], sizeof(listhack));
    memcpy(fileopts, &listwhat[i], sizeof(fileopts));
  } else {
    memset(listhack, 0, sizeof(listhack));
    memset(fileopts, 0, sizeof(fileopts));
  }
  return(j);
}

void SetItemData(menuitem *item, char *s, BYTE v, int p) {
  StrOverlap(&item->item[p], s);
  SetMenuData(*item, v);
}

int LevelHack(int t) {
fix_item ilst;
fix_info nlst;
fix_file flst;
  ilst.offs = 8; /* set_item.MissionMax */
  ilst.data = 0;
  nlst.stat = PH_ORG;
  nlst.file = 0;
  nlst.size = 1;
  nlst.list = &ilst;
  flst.name = "CREATURE.SET";
  flst.size = 24;
  if (t < 0) {
    ReadPatches(&flst, &nlst, 1);
    t = ilst.data;
    t = (t < 5) ? t : 4;
  } else {
    ilst.data = (t < 5) ? t : 4;
    nlst.stat = PH_MOD;
    SavePatches(&flst, &nlst, 1);
  }
  return(t);
}

int LoadHacks(void) {
int r, i;
  r = DetectVersion();
  mainfile[1 + r].flag ^= ITEM_HILITE | ITEM_GRAYED;
  DrawMenu(mainfile, NULL);
  if (r) {
    /* SHOCK.EXE */
    LoadPatches(fileopts, listhack, LIST_LEN(listhack));
    for (i = 0; i < LIST_LEN(listhack); i++) {
      SET_LITE(mainhack[1 + i + (i ? 1 : 0)], listhack[i].stat != PH_ORG);
    }
    /* CREATURE.SET */
    i = 1 + LevelHack(-1);
    SetItemData(&mainhack[8], mainmssn[i].item, i, 18);
  }
  return(r);
}

void SaveHacks(void) {
int i;
  /* SHOCK.EXE */
  for (i = 0; i < LIST_LEN(listhack); i++) {
    listhack[i].stat = ITS_LITE(mainhack[1 + i + (i ? 1 : 0)]) ? PH_MOD : PH_ORG;
  }
  SavePatches(fileopts, listhack, LIST_LEN(listhack));
  /* CREATURE.SET */
  LevelHack(GetMenuData(mainhack[8]) - 1);
}

int HackProc(int item) {
int i;
  if (item == 8) {
    DefMenuItem(mainmssn, GetMenuData(mainhack[item]) - 1);
    i = DrawMenu(mainmssn, NULL);
    if (i) {
      SetItemData(&mainhack[item], mainmssn[i].item, i, 18);
    }
  }
  return(item);
}

void MenuHacks(void) {
  DrawMenu(mainhack, HackProc);
}
